/**
 * Declaración de variables globales
 */
var color_principal = "#469990";
var color_resaltado = "#1C4D47";
var paleta_colores = ['#e6194B', '#3cb44b', '#4363d8', '#f58231', '#42d4f4','#ffe119', '#f032e6', '#fabed4', '#dcbeff', '#9A6324', '#911eb4', '#800000', '#000075', '#a9a9a9', '#469990', '#000000','#bfef45','#808000'];

var w = 1150; // Ancho del gráfico
var h = 400; // Alto del gráfico

var margin_x = 32;
var margin_y_up = 40;
var margin_y_down = 130;

// Definimos dominio y rango de los ejes
var x, y, xAxis, yAxis, svg, seleccionable;
var datosOriginales;
var tipo_seleccionado = "General"
var fechas = [];
var comunidades = [];
var comunidades_activas = [];
var datos;
var grafico = "barras";


function actualizarGraficoBarras(redibujar){

    // Si estamos creando el gráfico de barras por primera vez
    if(!redibujar){
        // Barra superior
        d3.select("#izquierda").append("div").text("Selecciona el periodo: ").append("select").attr("id","seleccionPeriodo");
        d3.select("#derecha").append("p").attr("class","enlace").on("click",ir_lineas).text("Ir al gráfico de líneas >");
        d3.select("#seleccionarTipo").style("margin-top","0px").style("padding-top","20px");

        // Definimos dominio y rango de los ejes
        x = d3.scaleBand().rangeRound([0,w]).padding(.1);
        y = d3.scaleLinear().range([h,0]);

        // Definimos los ejes
        xAxis = d3.axisBottom(x).ticks(5); // Defino la función eje X
        yAxis = d3.axisLeft(y).ticks(5); // Defino la función eje Y

        // Crear el svg para el gráfico
        svg = d3.select("#grafico").append("svg").attr("width", w + margin_x + margin_x) // Elementos svg
            .attr("height", h + margin_y_up + margin_y_down).append("g")
            .attr("transform", "translate(" + margin_x + "," + margin_y_up + ")").attr("position","absolute");

    }
    // Cargar opciones del seleccionable
    seleccionable = d3.select("#seleccionPeriodo").on("change",actualizarGraficoBarras);
    seleccionable.selectAll(".entrada").data(fechas).enter().append("option")
        .attr("class","entrada").text((d)=>(d));

    // Creamos una lista más pequeña con los datos que nos interesan
    let periodo_seleccionado = seleccionable.node().value;
    datos = datosOriginales.filter(eu => (eu.Periodo===periodo_seleccionado && eu.Tipo===tipo_seleccionado));

    // Creación ejes
    x.domain(datos.map((d)=>{return d.Comunidades}));
    y.domain([0,d3.max(datos,(d)=>{return (parseFloat(d.IPV));})]);

    svg.select(".x.axis").remove();
    svg.select(".y.axis").remove();
    svg.append("g").attr("class", "x axis").attr("transform", "translate(0," + h + ")").call(xAxis).selectAll("text")
        .style("text-anchor", "end").attr("width","5px").attr("dx", "-.8em")
        .attr("dy", ".15em")
        .attr("transform","rotate(-55)"); // Eje X
    svg.append("g").attr("class", "y axis").call(yAxis); // Eje Y

    if(document.getElementsByClassName("labels").length===0){
        let labels = svg.append("g").attr("class","labels");

        labels.append("text")
            // .attr("transform","rotate(-90)")
            .attr("y","-1.25em")
            .attr("dy",".71em")
            .style("text-anchor","end")
            .text("IPV");
    }

    // Se dibujan las barras.
    // Cambia la animación en función de si es una actualización o si se
    // está dibujando por primera vez.
    if(redibujar){
        svg.selectAll(".bar")
            .data(datos, (d)=>(d.Comunidades))
            .join(enter => enter.append("rect"),update=>update,exit=>exit.remove())
            .attr("class","bar")
            .attr("x",(d)=>{return x(d.Comunidades);})
            .attr("width",x.bandwidth())
            .transition().duration(1000)
            .attr("y",(d)=>{return y(parseFloat(d.IPV));})
            .attr("height",(d)=>{return (h - (y(parseFloat(d.IPV))));})
            .attr("fill", color_principal).on("end",definirActivadores);

    } else{
        svg.selectAll(".bar")
            .data(datos, (d)=>(d.Comunidades))
            .join(enter => enter.append("rect"),update=>update,exit=>exit.remove())
            .attr("class","bar")
            .attr("x",(d)=>{return x(d.Comunidades);})
            .attr("width",x.bandwidth()).attr("y",h).transition().duration(1000)
            .attr("y",(d)=>{return y(parseFloat(d.IPV));})
            .attr("height",(d)=>{return (h - (y(parseFloat(d.IPV))));})
            .attr("fill", color_principal).on("end",definirActivadores);
    }

}

function actualizarGraficoLineas(nueva_comunidad){

    // Si el cambio viene dado por el slider, hay que redibujar todo
    if(!nueva_comunidad){
        checkear_comunidades(comunidades_activas);

        svg.selectAll("circle").remove();
        svg.selectAll("line").remove();

    }

    // Nos quedamos solamente con los datos de las comunidades seleccionadas
    let datos = datosOriginales.filter(eu => comunidades_activas.includes(eu.Comunidades) && eu.Tipo===tipo_seleccionado);

    // Cálculo ejes
    x.domain(datos.map((d)=>{return d.Periodo}).reverse());
    y.domain([0,d3.max(datos,(d)=>{return (parseFloat(d.IPV));})]);

    svg.select(".x.axis").remove();
    svg.select(".y.axis").remove();

    svg.append("g").attr("class", "x axis").attr("transform", "translate(0," + h + ")").call(xAxis).selectAll("text")
        .style("text-anchor", "end").attr("width","5px").attr("dx", "-.8em")
        .attr("dy", ".15em")
        .attr("transform","rotate(-55)"); // Eje X

    svg.append("g").attr("class", "y axis").call(yAxis); // Eje Y

    if(document.getElementsByClassName("labels").length===0){
        let labels = svg.append("g").attr("class","labels");

        labels.append("text")
            // .attr("transform","rotate(-90)")
            .attr("y","-1.25em")
            .attr("dy",".71em")
            .style("text-anchor","end")
            .text("IPV");
    }

    // Dibujar
    for(let i=0;i<comunidades_activas.length;i++) {

        let comunidad = comunidades_activas[i];
        let datosComunidad = datos.filter(e => e.Comunidades === comunidad);
        let identificador = comunidades.indexOf(comunidad);

        // Si no hemos dibujado esa comunidad (es necesario para las líneas, los puntos con el .join lo hacen automático)

            svg.selectAll(".linea"+identificador).remove();

            // Dibujar línea
            for (let j = 0; j < datosComunidad.length - 1; j++) {
                svg.append("line").attr("class", "linea"+identificador).attr("x1", x(datosComunidad[j].Periodo))
                    .attr("y1", y(parseFloat(datosComunidad[j].IPV))).attr("x2", x(datosComunidad[j + 1].Periodo))
                    .attr("y2", y(parseFloat(datosComunidad[j + 1].IPV))).style("stroke", paleta_colores[comunidades.indexOf(comunidad)]).style("stroke-width", "4")
                    .style("opacity",0).transition().duration(500).style("opacity",100);
            }

            // Punto entre líneas (sirve para que no se vean las líneas como cachos)
            svg.selectAll(".punto_fake" + identificador)
                .data(datosComunidad, (d) => (d.Periodo))
                .join(enter => enter.append("circle"), update => update, exit => exit.remove())
                .attr("class", "punto_fake" + identificador)
                .attr("cx", (d) => {
                    return x(d.Periodo);
                })
                .attr("cy", (d) => {
                    return y(parseFloat(d.IPV));
                })
                .attr("r", "2")
                .attr("fill", paleta_colores[comunidades.indexOf(comunidad)])
                .style("opacity",0).transition().duration(500).style("opacity",100);

            // Punto invisible con mayor dimensión que el original: sirve para capturar el evento 'mouseover'
            svg.selectAll(".punto" + identificador)
                .data(datosComunidad, (d) => (d.Periodo))
                .join(enter => enter.append("circle"), update => update, exit => exit.remove())
                .attr("class", "punto" + identificador)
                .attr("cx", (d) => {
                    return x(d.Periodo);
                })
                .attr("cy", (d) => {
                    return y(parseFloat(d.IPV));
                })
                .attr("r", "12")
                .attr("fill", "transparent").on("mouseover", resaltarPunto).on("mouseout", restaurarPunto);
    }


}

/**
 * Transición entre gráficos.
 * Elimina el selector de comunidades, modifica
 * la barra superior y dibuja el gráfico de barras.
 */
function volver_barras(){

    grafico = "barras";

    d3.select("#barra_superior")
        .transition().duration(500)
        .style("opacity",0)
        .on("end",()=>{
            svg.html("");
            d3.select("#centro").html("");  // Limpiar el texto
            d3.select("#izquierda").append("div").text("Selecciona el periodo ").append("select").attr("id","seleccionPeriodo");
            d3.select("#derecha").append("p").attr("class","enlace").on("click",ir_lineas).text("Ir al gráfico de líneas >");  // Cambiar el texto
            d3.select("#selector_comunidades").remove();
            svg.html("");
            actualizarGraficoBarras(true);
            d3.select("#seleccionarTipo").style("margin-top","0px").style("padding-top","20px");
            d3.select("#barra_superior").transition().duration(1000).style("opacity",100);  // Reaparecer
        });


}

/**
 * Transición entre gráficos.
 * Cambia la barra superior, crea el selector de comunidades y en
 * función del evento, preselecciona la primera comunidad o selecciona
 * la barra pulsada por el usuario.
 * @param event
 */
function ir_lineas(event){

    grafico = "lineas";
    comunidades_activas = [];

    let elemento;
    
    if(event.originalTarget == null){
        elemento = event.srcElement.__data__;  // Se obtiene la comunidad de la columna
    } else{
        elemento = event.originalTarget.__data__;  // Se obtiene la comunidad de la columna
    }

    if(event.target.className==="enlace"){
        comunidades_activas.push(comunidades[0]);
    } else{
        comunidades_activas.push(elemento.Comunidades);
    }

    d3.select("#barra_superior")
        .transition().duration(500)
        .style("opacity",0)
        .on("end",()=>{
            d3.select("#izquierda").html("");  // Limpiar el texto
            d3.select("#derecha").html("");  // Limpiar el texto
            d3.select("#centro").append("p").attr("class","enlace").on("click",volver_barras).text("< Volver al gráfico de barras");  // Cambiar el texto
            svg.html("");
            let fila = d3.select("#fila");
            let columna_comunidades = fila.append("td").attr("id","selector_comunidades");
            columna_comunidades.style("vertical-align","top");
            columna_comunidades.append("div").style("text-decoration","underline").style("margin-bottom","7px").text("Selecciona comunidades: ");
            columna_comunidades.selectAll(".entrada")
                .data(comunidades).enter()
                .append("div").append("label").html((e,f)=>{return "<input type=checkbox class=\"entrada\" id=\""+"box"+f+"\" /> "+e;}).on("change",cambio_comunidad);
            columna_comunidades.append("div").attr("class","limpiar").on("click", limpiar).text("Limpiar selección");
            actualizarGraficoLineas(false);
            d3.select("#seleccionarTipo").transition().duration(500).style("margin-top","-50px");
            d3.select("#barra_superior").transition().duration(1000).style("opacity",100);  // Reaparecer
        });
}
function limpiar(){
    if(!comunidades_activas.empty) {
        comunidades_activas = [comunidades_activas[0]];

        d3.select("#selector_comunidades").remove();

        svg.html("");
        let fila = d3.select("#fila");
        let columna_comunidades = fila.append("td").attr("id", "selector_comunidades");
        columna_comunidades.style("vertical-align", "top");
        columna_comunidades.append("div").style("text-decoration", "underline").style("margin-bottom", "7px").text("Selecciona comunidades: ");
        columna_comunidades.selectAll(".entrada")
            .data(comunidades).enter()
            .append("div").append("label").html((e, f) => {
            return "<input type=checkbox class=\"entrada\" id=\"" + "box" + f + "\" /> " + e;
        }).on("change", cambio_comunidad);
        columna_comunidades.append("div").attr("class", "limpiar").on("click", limpiar).text("Limpiar selección");
        actualizarGraficoLineas(false);
    }
}
/**
 * Marca las comunidades indicadas en comunidades_checkear.
 * @param comunidades_checkear  Las comunidades a marcar.
 */
function checkear_comunidades(comunidades_checkear){
    comunidades_checkear.forEach((elem)=>{
        let elementos = d3.selectAll("label").filter((d)=>elem===d);
        elementos.select("input").attr("checked",true);
        elementos.select("span").remove();
        elementos.append("span").style("color",paleta_colores[comunidades.indexOf(elem)]).text(" ▪");
    });
}

/**
 * Handler ante el cambio en la selección de comunidad.
 * Pone un pequeño cuadrado al lado del nombre con el color a modo de leyenda.
 * Dibuja el gráfico.
 * @param event
 */
function cambio_comunidad(event){

    let comunidad = this.__data__;

    if(event.target.checked){
        comunidades_activas.push(comunidad);
        d3.select(this).append("span").style("color",paleta_colores[comunidades.indexOf(comunidad)]).text(" ▪");
    } else{

        comunidades_activas.splice(comunidades_activas.indexOf(comunidad),1);
        let identificador = comunidades.indexOf(comunidad);

        svg.selectAll(".linea" + identificador).remove();
        svg.selectAll(".punto_fake" + identificador).remove();
        svg.selectAll(".punto" + identificador).remove();


        d3.select(this).select("span").remove();
    }

    actualizarGraficoLineas(true);
}

/**
 * Handler para cuando se pasa por encima de una barra.
 * Oscurece la barra, indica el valor del IPV y vincula la función ir_lineas.
 * @param event
 */
function resaltarColumna(event){

        let columna = d3.select(this);
        let comunidad;

        if(event.originalTarget == null){
            comunidad = event.srcElement.__data__.Comunidades;  // Se obtiene la comunidad de la columna
        } else{
            comunidad = event.originalTarget.__data__.Comunidades;  // Se obtiene la comunidad de la columna
        }
        
        columna.style("fill", color_resaltado).style("cursor","pointer").on("click",ir_lineas); // Se cambia el color de la columna
        let dato = datos.filter((d) => (d.Comunidades === comunidad))[0];    // Se obtiene el IPV

        let equis = parseFloat(columna.attr("x")) + (parseFloat(columna.attr("width")) / 2); // Posición del texto

        // Se dibuja el valor del ipv encima de la barra
        svg.append("text")
            .attr("id", "valor")
            .attr("class", "numero_grafico")
            .text(dato.IPV)
            .attr("x", equis)
            .attr("y", parseFloat(columna.attr("y")) - 10)
            .style("font-size", columna.attr("width") / 5 + "pt")
            .attr("text-anchor", "middle");

}

/**
 * Devuelve la columna a su estado original
 * @param event
 */
function restaurarColumna(event){
    d3.select("#valor").remove();   // Quitar el texto
    d3.select(this).style("fill",color_principal);  // Devolver el color original
}

/**
 * Handler para cuando se pasa por encima de un punto del gráfico de líneas.
 * Resalta el punto aumentando su tamaño y muestra el IPV en una caja cercana al punto.
 * @param event
 */
function resaltarPunto(event) {
    let punto = d3.select(this);
    let elemento;
    
    if(event.originalTarget == null){
        elemento = event.srcElement.__data__;  // Se obtiene la comunidad de la columna
    } else{
        elemento = event.originalTarget.__data__;  // Se obtiene la comunidad de la columna
    }

    let comunidad = elemento.Comunidades;
    let indice = elemento.IPV;

    punto.attr("r", 5).attr("fill", paleta_colores[comunidades.indexOf(comunidad)]);

    // Crear el texto para saber cuanto ocupa
    let x = punto.attr("cx");
    let y = parseFloat(punto.attr("cy")) - 20
    let texto = svg.append("text").attr("x", x).attr("y", y).attr("class", "numero_grafico").text(indice);

    let punto_js = document.getElementsByClassName("numero_grafico")[0];
    let width = punto_js.getBBox().width;
    let height = punto_js.getBBox().height;
    let margin_x = 20;

    texto.remove()
    svg.append("rect")
        .attr("id","fondo_texto")
        .attr("x", x-(margin_x/2))
        .attr("y", y - height)
        .attr("rx",5)
        .attr("width",width+margin_x)
        .attr("height",height+10)
        .attr("fill",color_principal);

    svg.append("text").attr("x", punto.attr("cx")).attr("y", parseFloat(punto.attr("cy")) - 20).attr("class", "numero_grafico").text(indice);
}

/**
 * Devuelve el punto a su estado original cuando se mueve
 * el ratón fuera del punto.
 * @param event
 */
function restaurarPunto(event){
    let punto = d3.select(this);

    punto.attr("r",12).attr("fill","transparent");
    d3.select(".numero_grafico").remove();
    d3.select("#fondo_texto").remove();
}

/**
 * Define las funciones ante eventos para columnas.
 */
function definirActivadores(){
    svg.selectAll(".bar").on("mouseover",resaltarColumna).on("mouseout",restaurarColumna);
}

// Cargar los datos, reconocer los periodos y las comunidades.
d3.dsv(";","./data/data.csv").then((data)=>{

    datosOriginales = data;

    data.forEach(element => {
        if(!fechas.includes(element.Periodo)){
            fechas.push(element.Periodo);
        }
        if(!comunidades.includes(element.Comunidades)){
            comunidades.push(element.Comunidades);
        }
    });
    //actualizarGraficoLineas(false);
    actualizarGraficoBarras(false);
});
